@@ -27,7 +27,11 @@ from .runners import (
     DefaultEditor,
     DefaultRunners,
 )
-from .site import Sites, Site, Synchronize, MinioFileUploader, MinioFileDownloader
+from .site import (
+    Synchronize,
+    WorkingSites, ExchangeSites,
+    MinioFileUploader, MinioFileDownloader,
+)
 from .kitsu import KitsuAPIWrapper
 
 
@@ -130,11 +134,11 @@ class SynchronizeFiles(Synchronize):
             return
         
         current_site = self.root().project().get_current_site()
-        nb_waiting_jobs = len(current_site.queue.jobs(status="WAITING"))
+        nb_waiting_jobs = len(current_site.get_jobs(status="WAITING"))
 
         super(SynchronizeFiles, self).run(button)
 
-        nb_processed_jobs = nb_waiting_jobs - len(current_site.queue.jobs(status="WAITING"))
+        nb_processed_jobs = nb_waiting_jobs - len(current_site.get_jobs(status="WAITING"))
 
         if nb_processed_jobs == nb_waiting_jobs:
             self.result.message.set((
@@ -163,12 +167,12 @@ class GotoBookmarks(flow.Action):
         )
 
 
-class SiteChoiceValue(flow.values.ChoiceValue):
+class WorkingSiteChoiceValue(flow.values.ChoiceValue):
 
     _parent = flow.Parent()
 
     def choices(self):
-        return self.root().project().admin.sites.mapped_names()
+        return self.root().project().get_working_sites().mapped_names()
 
 
 class RequestRevisionsAs(flow.Action):
@@ -176,10 +180,10 @@ class RequestRevisionsAs(flow.Action):
     path = flow.SessionParam("").ui(
         placeholder="Revision oid pattern (e.g., /siren/films/siren/sequences/sq27/shots/*/departments/misc/files/*/history/revisions/v001)"
     )
-    requesting_site_name = flow.Param("default", SiteChoiceValue).watched().ui(
+    requesting_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
         label="Requesting site"
     )
-    requested_site_name = flow.Param("default", SiteChoiceValue).watched().ui(
+    requested_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
         label="Site to query"
     )
 
@@ -290,7 +294,7 @@ class Synchronization(flow.Object):
 
     def summary(self):
         current_site = self._project.get_current_site()
-        nb_waiting_jobs = len(current_site.queue.jobs(status="WAITING"))
+        nb_waiting_jobs = len(current_site.get_jobs(status="WAITING"))
         
         if nb_waiting_jobs > 0:
             return (
@@ -352,33 +356,27 @@ class ProjectSettings(flow.Object):
     project_thumbnail = flow.Param().ui(editor='textarea')
 
 
-class Admin(flow.Object):
+class ExchangeSiteChoiceValue(flow.values.ChoiceValue):
 
-    ICON = ("icons.gui", "team-admin")
+    def choices(self):
+        return self.root().project().get_exchange_sites().mapped_names()
 
-    _project = flow.Parent()
+
+class MultisiteConfig(flow.Object):
+
+    ICON = ('icons.gui', 'sitemap')
 
     current_site_name = flow.Computed(cached=True)
     root_dir = flow.Computed(cached=True)
-    # kitsu_server_url = flow.Param("").watched()
-
-    project_settings = flow.Child(ProjectSettings)
-    settings = flow.Child(ContextualView)
-    users = flow.Child(Users)
-    kitsu = flow.Child(KitsuConfig)
-    
-    user_environment = flow.Child(UserEnvironment).ui(expanded=False)
-    default_applications = flow.Child(DefaultRunners).ui(expanded=False).injectable()
-
-    sites = flow.Child(Sites)
-    process_jobs = flow.Child(Synchronize)
-    login_page = flow.Child(LoginPage).ui(hidden=True)
+    exchange_site_name = flow.Param("default", ExchangeSiteChoiceValue).watched().ui(
+        editable=False
+    )
+    working_sites = flow.Child(WorkingSites).ui(label="Working sites")
+    exchange_sites = flow.Child(ExchangeSites).ui(label="Exchange sites")
 
     def child_value_changed(self, child_value):
-        if child_value is self.kitsu_server_url:
-            self._project.update_kitsu_host(child_value.get())
-            self._project.kitsu_id.touch()
-            self._project.kitsu_url.touch()
+        if child_value is self.exchange_site_name:
+            self.touch()
 
     def compute_child_value(self, child_value):
         if child_value == self.root_dir:
@@ -391,38 +389,51 @@ class Admin(flow.Object):
                 root_dir = os.environ["ROOT_DIR"]
             else:
                 # Otherwise, get current site's root dir
-                root_dir = self.sites[self.current_site_name.get()].root_folder.get()
+                root_dir = self.working_sites[self.current_site_name.get()].root_folder.get()
             child_value.set(root_dir)
         elif child_value is self.current_site_name:
-            site_name = None
-            site_name_var = "KABARET_SITE_NAME"
-
-            if site_name_var in os.environ:
-                site_name = os.environ[site_name_var]
+            if "KABARET_SITE_NAME" in os.environ:
+                site_name = os.environ["KABARET_SITE_NAME"]
             else:
-                cluster_name = os.environ["KABARET_CLUSTER_NAME"]
+                site_name = os.environ["KABARET_CLUSTER_NAME"]
+
+            if not self.working_sites.has_mapped_name(site_name):
+                self.root().session().log_warning((
+                    f"Site {site_name} not found in project working sites. "
+                    "Falling back to default."
+                ))
+                site_name = "default"
 
-                if cluster_name in self.sites.mapped_names():
-                    site_name = cluster_name
-                else:
-                    site_name = "default"
             child_value.set(site_name)
-        # elif child_value is self.current_user_id:
-        #     # Check env
-        #     if "USER_NAME" in os.environ:
-        #         child_value.set(os.environ["USER_NAME"])
-        #         return
-            
-        #     # Check user file
-        #     user_file = os.path.join(self._project.user_settings_folder(), "user.json")
-        #     if os.path.exists(user_file):
-        #         with open(user_file, "r") as f:
-        #             user_config = json.load(f)
-        #             child_value.set(user_config["username"])
-        #             return
 
-        #     # Return local user name otherwise
-        #     child_value.set(getpass.getuser())
+    def summary(self):
+        exchange_site = self.exchange_sites[self.exchange_site_name.get()]
+        
+        if not exchange_site.configured.get():
+            return (
+                "<font color=#D66700>"
+                "Exchange site <b>%s</b> not completely configured."
+                "</font>" % exchange_site.name()
+            )
+
+
+class Admin(flow.Object):
+
+    ICON = ("icons.gui", "team-admin")
+
+    _project = flow.Parent()
+
+    project_settings = flow.Child(ProjectSettings)
+    settings = flow.Child(ContextualView)
+    users = flow.Child(Users)
+    kitsu = flow.Child(KitsuConfig)
+    
+    user_environment = flow.Child(UserEnvironment).ui(expanded=False)
+    default_applications = flow.Child(DefaultRunners).ui(expanded=False).injectable()
+
+    multisites = flow.Child(MultisiteConfig).ui(label="Multi-site configuration")
+    process_jobs = flow.Child(Synchronize)
+    login_page = flow.Child(LoginPage).ui(hidden=True)
 
 
 class Project(flow.Object):
@@ -443,7 +454,7 @@ class Project(flow.Object):
         '''
         alternative can be used if root_dir.get() returns None
         '''
-        root_dir = self.admin.root_dir.get()
+        root_dir = self.admin.multisites.root_dir.get()
         if root_dir is None and alternative != None:
             root_dir = alternative
         return root_dir
@@ -470,20 +481,40 @@ class Project(flow.Object):
         Returns the site within which the current
         session is beeing run.
         """
-        return self.admin.sites[
-            self.admin.current_site_name.get()
-        ]
+        return self.get_working_site(
+            self.admin.multisites.current_site_name.get()
+        )
+    
+    def get_working_site(self, name):
+        """
+        Returns the working site of the project
+        with the given name.
+        """
+        return self.get_working_sites()[name]
     
     def get_exchange_site(self):
         """
         Returns the first exchange site found in
         project's registered site list.
         """
-        for site in self.admin.sites.mapped_items():
-            if site.site_type.get() == "Exchange":
-                return site
+        return self.get_exchange_sites()[
+            self.admin.multisites.exchange_site_name.get()
+        ]
 
-        return None
+    def get_working_sites(self):
+        return self.admin.multisites.working_sites
+
+    def get_exchange_sites(self):
+        return self.admin.multisites.exchange_sites
+    
+    def get_site_names(self, short_names=False):
+        working_sites = self.get_working_sites()
+        exchange_sites = self.get_exchange_sites()
+
+        if short_names:
+            return working_sites.short_names() + exchange_sites.short_names()
+
+        return working_sites.mapped_names() + exchange_sites.mapped_names()
 
     def is_admin(self, username):
         try:
