@@ -183,28 +183,38 @@ class Revision(flow.Object):
 
         return self.name() == self.user.get()
 
-    def get_sync_status(self, site_name=None):
+    def get_sync_status(self, site_name=None, exchange=False):
         """
         Returns revision's status on the site identified
-        by the given name.
+        by the given name, or the project's exchange site
+        if `exchange` is True.
 
         If site_name is None, this method returns its status
         on the current site.
         """
+        if exchange:
+            exchange_site_name = self.root().project().get_exchange_site().name()
+            return self.sync[exchange_site_name].status.get()
+
         if not site_name:
-            site_name = self.root().project().admin.current_site_name.get()
+            site_name = self.root().project().get_current_site().name()
         return self.sync[site_name].status.get()
 
-    def set_sync_status(self, status, site_name=None):
+    def set_sync_status(self, status, site_name=None, exchange=False):
         """
         Sets revision's status on the site identified
-        by the given name to the given status.
+        by the given name, or the project's exchange site
+        if `exchange` is True, to the given status.
 
         If site_name is None, this method sets its status
         on the current site.
         """
+        if exchange:
+            exchange_site_name = self.root().project().get_exchange_site().name()
+            return self.sync[exchange_site_name].status.get()
+
         if not site_name:
-            site_name = self.root().project().admin.current_site_name.get()
+            site_name = self.root().project().get_current_site().name()
         self.sync[site_name].status.set(status)
 
     def get_last_edit_time(self):
@@ -302,10 +312,8 @@ class Revisions(flow.Map):
         columns = ["Revision", "Created by", "When"]
         
         if self._show_sync_statuses.get():
-            if self._enable_short_names.get():
-                columns += self.root().project().admin.sites.short_names()
-            else:
-                columns += self.root().project().admin.sites.mapped_names()
+            columns += self.root().project().get_site_names(short_names=self._enable_short_names.get())
+        
         columns.append("Comment")
 
         return columns
@@ -346,10 +354,7 @@ class Revisions(flow.Map):
         )
 
         if self._show_sync_statuses.get():
-            if self._enable_short_names.get():
-                names = self.root().project().admin.sites.short_names()
-            else:
-                names = self.root().project().admin.sites.mapped_names()
+            names = self.root().project().get_site_names(short_names=self._enable_short_names.get())
             
             d = dict.fromkeys(names, "")
             row.update(d)
@@ -541,6 +546,9 @@ class SeeRevisionAction(flow.Action):
     _file = flow.Parent()
     revision_name = flow.Param(None, RevisionsChoiceValue).ui(label="Revision")
 
+    def allow_context(self, context):
+        return False
+
     def get_buttons(self):
         self.message.set("<h3>Choose a revision to open</h3>")
 
@@ -766,7 +774,7 @@ class PublishFileAction(LaunchSessionWorker):
         head = self._file.get_head_revision()
         current_site = self.root().project().get_current_site()
 
-        upload_job = current_site.queue.submit_job(
+        upload_job = current_site.get_queue().submit_job(
             emitter_oid=head.oid(),
             user=self.root().project().get_user(),
             studio=current_site.name(),
@@ -991,6 +999,9 @@ class SearchExistingRevisions(flow.Action):
 
     def needs_dialog(self):
         return False
+    
+    def allow_context(self, context):
+        return False
 
     def run(self, button):
         folders = [
@@ -1019,7 +1030,11 @@ class RenderBlenderPlayblastAction(OpenWithBlenderAction):
 
     def get_buttons(self):
         self.revision_name.revert_to_default()
-        return ["Render", "Publish first", "Cancel"]
+
+        if self._file.has_working_copy(from_current_user=True):
+            return ["Render", "Publish first", "Cancel"]
+        
+        return ["Render", "Cancel"]
 
     def needs_dialog(self):
         return True
@@ -1072,7 +1087,7 @@ class RenderBlenderPlayblastAction(OpenWithBlenderAction):
         python_expr = """import bpy
 bpy.ops.lfs.playblast(do_render=True, filepath='%s', studio='%s', project='%s', sequence='s%04i', scene='%s')""" % (
             self.output_path,
-            self.root().project().admin.current_site_name.get(),
+            self.root().project().get_current_site().name(),
             project_name,
             self._sequence_number_from_name(file_settings["sequence"]),
             file_settings["shot"],
@@ -1144,7 +1159,7 @@ class PublishAndRenderPlayblastAction(PublishFileAction):
         return (
             context
             and self._file.format.get() == "blend"
-            and not self._file.is_empty()
+            and self._file.has_working_copy(from_current_user=True)
         )
 
     def run(self, button):
@@ -1159,6 +1174,31 @@ class PublishAndRenderPlayblastAction(PublishFileAction):
         return self._file.render_playblast.run("Render")
 
 
+class RequestTrackedFileAction(flow.Action):
+
+    _file = flow.Parent()
+    _files = flow.Parent(2)
+
+    def needs_dialog(self):
+        return False
+    
+    def allow_context(self, context):
+        return False
+    
+    def run(self, button):
+        head = self._file.get_head_revision()
+        exchange_site_name = self.root().project().get_exchange_site().name()
+
+        if not head or head.get_sync_status() != "NotAvailable" or head.get_sync_status(site_name=exchange_site_name) != "Available":
+            return
+        
+        head.request.requesting_site_name.set(
+            self.root().project().get_current_site().name()
+        )
+        head.request.run(None)
+        self._files.touch()
+
+
 class TrackedFile(File):
 
     ICON = ("icons.gui", "text-file-1")
@@ -1189,6 +1229,7 @@ class TrackedFile(File):
     lock_action = flow.Child(LockAction).ui(label="Lock")
     unlock_action = flow.Child(UnlockAction).ui(label="Unlock")
     search_existing_revisions = flow.Child(SearchExistingRevisions)
+    request = flow.Child(RequestTrackedFileAction)
     
     with flow.group("Playblast"):
         render_playblast = flow.Child(RenderBlenderPlayblastAction)
@@ -1269,8 +1310,13 @@ class TrackedFile(File):
     def get_revisions(self):
         return self.history.revisions
 
-    def has_revision(self, name):
-        return name in self.history.revisions.mapped_names()
+    def has_revision(self, name, sync_status=None):
+        exists = (name in self.history.revisions.mapped_names())
+
+        if exists and sync_status:
+            exists = exists and (self.history.revisions[name].get_sync_status() == sync_status)
+        
+        return exists
 
     def is_empty(self):
         return not bool(len(self.get_revisions()))
@@ -1945,7 +1991,7 @@ class FileSystemMap(flow.Map):
         return flow.injection.injectable(FileSystemItem)
 
     def columns(self):
-        return ["Lock", "Name", "Last comment"]
+        return ["Lock", "Name", "Last comment", "Latest"]
 
     def _fill_row_cells(self, row, item):
         if isinstance(item, File):
@@ -1963,6 +2009,12 @@ class FileSystemMap(flow.Map):
             row["Last comment"] = "-"
 
         row['Last edit'] =  timeago.format(datetime.datetime.fromtimestamp(item.get_last_edit_time()), datetime.datetime.now())
+        
+        if isinstance(item, TrackedFile):
+            head = item.get_head_revision()
+            row["Latest"] = head.name() if head else ""
+        else:
+            row["Latest"] = "-"
 
     def _fill_row_style(self, style, item, row):
         style["icon"] = ("icons.libreflow", "blank")
@@ -1975,6 +2027,31 @@ class FileSystemMap(flow.Map):
                 else:
                     style["icon"] = ("icons.libreflow", "padlock_red")
 
+            head = item.get_head_revision()
+            exchange_site_name = self.root().project().get_exchange_site().name()
+
+            if head:
+                head_sync_status = head.get_sync_status()
+
+                if head_sync_status == "NotAvailable":
+                    if head.get_sync_status(site_name=exchange_site_name) == "Available":
+                        sync_icon = ('icons.libreflow', 'downloadable')
+                    else:
+                        sync_icon = ('icons.libreflow', 'blank')
+                    
+                    style["Latest_foreground-color"] = "#777"
+                elif head_sync_status == "Requested":
+                    sync_icon = ('icons.libreflow', 'waiting')
+                    style["Latest_foreground-color"] = "#777"
+                else:
+                    sync_icon = ('icons.libreflow', 'blank')
+            else:
+                sync_icon = ('icons.libreflow', 'blank')
+            
+            style["Latest_icon"] = sync_icon
+            style["Latest_activate_oid"] = item.request.oid()
+            style["Last comment_activate_oid"] = item.show_history.oid()
+
         if isinstance(item, File):
             default_applications = self.root().project().admin.default_applications
             try:
@@ -1987,7 +2064,8 @@ class FileSystemMap(flow.Map):
                 except AttributeError:
                     pass
 
-        style["activate_oid"] = item.open.oid()
+        style["Name_activate_oid"] = item.open.oid()
+        
 
     def _parse_filename(self, filename, **params):
         formatted_name, extension = tuple(filename.split("."))
